import git
import logging
import os
import os.path
import pathlib
import sys

def weblateComponentFile(filename, tree):
    """is this file a weblate component"""
    if not filename.startswith("wiki/src"):
        return False

    basename, *extensions = os.path.basename(filename).split(".")
    if extensions and extensions[-1] == "po":
        return False

    detectionFile = "{}.{}.po".format(os.path.splitext(filename)[0],LANG)
    try:
        tree/detectionFile
        return True
    except KeyError:
        return False
    return False

# Format logging
ch = logging.StreamHandler()
ch.setLevel(logging.INFO)  # Change to DEBUG if you wnat to see debug messages

formatter = logging.Formatter('%(levelname)s: %(message)s')
ch.setFormatter(formatter)

logger = logging.getLogger('')
logger.setLevel(logging.DEBUG)
logger.addHandler(ch)

# DJANGO/WEBLATE setup to be able to change stuff within weblate
sys.path.insert(0,"/usr/local/share/weblate")
os.environ["DJANGO_SETTINGS_MODULE"] = "weblate.settings"
os.environ["DJANGO_IS_MANAGEMENT_COMMAND"] = "1"

from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()

from weblate.trans import models
from weblate.trans.management.commands import import_project
from django.utils.text import slugify

def runImportProject(directory):
    """ Use import_project as used for inital creation of components
        see /usr/local/sbin/weblate-setup-project.sh
    """
    args=["--component-regex", str(directory/"(?P<name>.*)\.(?P<language>.+)\.po"),
        "--name-template", str(directory/"%s.*.po"),
        PROJECT,
        REPO,
        "master",
        str(directory/"**.*.po"),
        ]

    command = import_project.Command()
    parser = command.create_parser("",None)
    options = parser.parse_args(args=args)
    command.handle(**vars(options))

# weblate project name
PROJECT =  models.Project.objects.first().name

# repo path ( should be start with weblate:// )
REPO = models.SubProject.objects.first().repo

# Where is the git repository located?
PATH = pathlib.Path("/var/lib/weblate/repositories/vcs/tails/wikisrcindexpo")

# One langague that is activated and handled within the main git
LANG = "fr"

if not REPO.startswith("weblate://"):
    logger.error("We need a weblate:// url for searching for new components and not '{}'.".format(REPO))
    sys.exit(-1)

repo = git.Repo(str(PATH))

# fetch first, to see what we need to update
repo.remotes.origin.fetch()

# the object to orgin master branch
remote_master = repo.remotes.origin.refs.master

# create a diff
# if there more than l files changed, git disables the detection of renames, as the algorithm is O(n^2)
# but we need this, otherwise we do not detect renames (see man git-diff)

diff = repo.index.diff(remote_master.commit,"wiki/src/", l=10000)

# perform pull
logger.info("HEAD before pull is at {}".format(repo.head.commit.hexsha))
repo.remotes.origin.pull()

for deleted_file in diff.iter_change_type('D'):
    if not weblateComponentFile(deleted_file.a_path, tree=repo.tree()):
        logger.debug("deleted file (ignoring): {}".format(deleted_file.a_path))
        continue
    filemask = os.path.splitext(deleted_file.a_path)[0] + ".*.po"
    logger.info("deleted component: {}".format(filemask))
    subproject = models.SubProject.objects.filter(filemask=filemask).first()
    if subproject is None:
        logger.warning("\t... can't find component :(")
        continue
    if not subproject.repo.startswith("weblate://"):
        logger.warning("Do not delete {}, because it has no weblate url {}.".format(filemask, subproject.repo))
        continue
    subproject.delete()

# Handle added files
# as we rescan the whole dictonary anyways, we only need this once per dictonary

rescanDirs=set()
for added_file in diff.iter_change_type('A'):
    if not weblateComponentFile(added_file.a_path, tree=remote_master.commit.tree):
        logger.debug("added file (ignoring): {}".format(added_file.a_path))
        continue
    filemask = os.path.splitext(added_file.a_path)[0] + ".*.po"
    logger.info("added component: {}".format(filemask))
    fname = pathlib.Path(added_file.a_path)
    dirname = fname.parent
    rescanDirs.add(dirname)

for directory in rescanDirs:

    logger.info("searching for new components in {}".format(directory))
    runImportProject(directory)

# Handle renamed files
for renamed_file in diff.iter_change_type('R'):
    if not weblateComponentFile(renamed_file.rename_to, tree=remote_master.commit.tree):
        logger.debug("renamed file (ignoring) {} -> {}".format(renamed_file.rename_from,renamed_file.rename_to))
        continue
    filemaskOld = os.path.splitext(renamed_file.rename_from)[0] + ".*.po"
    filemaskNew = os.path.splitext(renamed_file.rename_to)[0] + ".*.po"
    logger.info("renamed component {} -> {}".format(filemaskOld, filemaskNew))
    subproject = models.SubProject.objects.filter(filemask=filemaskOld).first()
    if subproject is None:
        logger.warning("\t... can't find component :(")
        continue
    if models.SubProject.objects.filter(filemask=filemaskNew).first():
        logger.warning("\t... renamed component exists already.")
        continue
    subproject.filemask = filemaskNew
    subproject.name = filemaskNew
    subproject.slug = slugify(subproject.name)
    subproject.save() # Triggers create_translations and update the translations.

